@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.28'; # VERSION
+our $VERSION = '0.29'; # VERSION
 
 # ABSTRACT: parse XLSX files
 
@@ -67,7 +67,7 @@ sub parse {
         $workbook->{File} = $file;
     }
     else {
-        die "Argument to 'new' must be a filename, open filehandle, or scalar ref";
+        die "First argument to 'parse' must be a filename, open filehandle, or scalar ref";
     }
 
     return $self->_parse_workbook($zip, $workbook, $formatter);
@@ -152,6 +152,16 @@ sub _parse_workbook {
           );
           $sheet->{SheetHidden} = 1 if defined $_->att('state') and $_->att('state') eq 'hidden';
           $self->_parse_sheet($sheet, $files->{sheets}{$idx});
+
+          # Do we have a rels for for this sheet?
+          if (
+            $files->{sheets_rels}
+            && $files->{sheets_rels}{$idx}
+          ) {
+            # Yes - now parse the rels to extract the hyperlinks
+            $self->_parse_sheet_links($sheet, $files->{sheets}{$idx}, $files->{sheets_rels}{$idx});
+          }
+
           ($sheet)
         } else {
           ()
@@ -364,7 +374,7 @@ sub _parse_sheet {
                     }
                     elsif ($type eq 'n') {
                         $long_type = 'Numeric';
-                        $val = defined($val) ? 0+$val : undef;
+                        $val = defined($val) && $val ne '' ? 0+$val : undef;
                     }
                     elsif ($type eq 'd') {
                         $long_type = 'Date';
@@ -385,6 +395,7 @@ sub _parse_sheet {
 
                     my $format_idx = $cell->att('s') || 0;
                     my $format = $sheet->{_Book}{Format}[$format_idx];
+                    my $formatstr = $sheet->{_Book}{FormatStr}{$format->{FmtIdx}};
                     die "unknown format $format_idx" unless $format;
 
                     # see the list of built-in formats below in _parse_styles
@@ -394,6 +405,10 @@ sub _parse_sheet {
                         $long_type = 'Date';
                     }
 
+                    if ($formatstr =~ /\b(mmm|m|d|yy|h|hh|mm|ss)\b/i) {
+                        $long_type = 'Date';
+                    }
+
                     my $formula = $cell->first_child('s:f');
                     my $cell = Spreadsheet::ParseExcel::Cell->new(
                         Val      => $val,
@@ -447,6 +462,102 @@ sub _parse_sheet {
 
 }
 
+sub _parse_sheet_links {
+    my $self = shift;
+    my ($sheet, $sheet_file, $rels_file) = @_;
+
+    # First we need to parse the hyperlinks out of the rels XML
+    my $rels;
+
+    my $rels_xml = XML::Twig->new(
+        twig_roots => {
+            'Relationships/Relationship' => sub {
+                my $twig = shift;
+                my $relationship = shift;
+
+                if (
+                  $relationship->att('Type') eq 'http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink'
+                  && $relationship->att('TargetMode') eq 'External'
+                ) {
+                    # Store the target URL in a hash by relationship id
+                    $rels->{$relationship->att('Id')} = $relationship->att('Target');
+                }
+
+                $twig->purge;
+            },
+        },
+    );
+
+    # Run the parser
+    $rels_xml->parse($rels_file);
+
+    # Now iterate over the sheet XML again, this time processing hyperlink entries
+    my $sheet_xml = XML::Twig->new(
+        twig_roots => {
+            'hyperlinks/hyperlink' => sub {
+                my $twig = shift;
+                my $hyperlink = shift;
+
+                # Work out our row and column
+                my ($row, $col) = $self->_cell_to_row_col($hyperlink->att('ref'));
+
+                # Get the cell
+                my $cell = $sheet->{Cells}[$row][$col];
+
+                # Do I have a cell?
+                unless ($cell) {
+                    # No - just create an empty value for now
+                    $cell = $sheet->{Cells}[$row][$col] = Spreadsheet::ParseExcel::Cell->new();
+                }
+
+                # Is this an external hyperlink I've parsed from the rels?
+                if (
+                    $hyperlink->att('r:id')
+                    && $rels
+                    && $rels->{$hyperlink->att('r:id')}
+                ) {
+                    # Yes - Check if we need to frig our destination a bit
+                    my $destination_url = sprintf(
+                        '%s%s%s',
+                        $rels->{$hyperlink->att('r:id')},
+                        $hyperlink->att('location') ? '#' : '',
+                        $hyperlink->att('location') || '',
+                    );
+
+                    # Add the hyperlink
+                    $cell->{Hyperlink} = [
+                        $hyperlink->att('display') || $cell->{_Value} || undef, # Description
+                        $destination_url, # Target
+                        undef, # Target Frame
+                        $row, # Start Row
+                        $row, # End Row
+                        $col, # Start Column
+                        $col, # End Column
+                    ];
+                } else {
+                    # This is an internal hyperlink
+
+                    # Add the hyperlink
+                    $cell->{Hyperlink} = [
+                        $hyperlink->att('display') || $cell->{_Value} || undef, # Description
+                        $hyperlink->att('location'), # Target
+                        undef, # Target Frame
+                        $row, # Start Row
+                        $row, # End Row
+                        $col, # Start Column
+                        $col, # End Column
+                    ];
+                }
+
+                $twig->purge;
+            },
+        },
+    );
+
+    # Now parse the XML
+    $sheet_xml->parse( $sheet_file );
+}
+
 sub _get_text_and_rich_font_by_cell {
     my $self = shift;
     my ($si, $theme_colors) = @_;
@@ -466,69 +577,74 @@ sub _get_text_and_rich_font_by_cell {
 
     my $string_text = '';
     my @rich_font_by_cell;
-    my @nodes_r = $si->find_nodes('.//s:r');
-    if (@nodes_r > 0) {
-        for my $chunk (map { $_->children } @nodes_r) {
-            my $string_length = length($string_text);
-            if ($chunk->name eq 's:t') {
-                if (!@rich_font_by_cell) {
+    for my $subnode ($si->children) {
+        if ($subnode->name eq 's:t') {
+            $string_text .= $subnode->text;
+        }
+        elsif ($subnode->name eq 's:r') {
+            for my $chunk ($subnode->children) {
+                my $string_length = length($string_text);
+                if ($chunk->name eq 's:t') {
+                    if (!@rich_font_by_cell) {
+                        push @rich_font_by_cell, [
+                            $string_length,
+                            Spreadsheet::ParseExcel::Font->new(%default_font_opts)
+                        ];
+                    }
+                    $string_text .= $chunk->text;
+                }
+                elsif ($chunk->name eq 's:rPr') {
+                    my %format_text = %default_font_opts;
+                    for my $node_format ($chunk->children) {
+                        if ($node_format->name eq 's:sz') {
+                            $format_text{Height} = $node_format->att('val');
+                        }
+                        elsif ($node_format->name eq 's:color') {
+                            $format_text{Color} = $self->_color(
+                                $theme_colors,
+                                $node_format
+                            );
+                        }
+                        elsif ($node_format->name eq 's:rFont') {
+                            $format_text{Name} = $node_format->att('val');
+                        }
+                        elsif ($node_format->name eq 's:b') {
+                            $format_text{Bold} = 1;
+                        }
+                        elsif ($node_format->name eq 's:i') {
+                            $format_text{Italic} = 1;
+                        }
+                        elsif ($node_format->name eq 's:u') {
+                            $format_text{Underline} = 1;
+                            if (defined $node_format->att('val')) {
+                                $format_text{UnderlineStyle} = 2;
+                            } else {
+                                $format_text{UnderlineStyle} = 1;
+                            }
+                        }
+                        elsif ($node_format->name eq 's:strike') {
+                            $format_text{Strikeout} = 1;
+                        }
+                        elsif ($node_format->name eq 's:vertAlign') {
+                            if ($node_format->att('val') eq 'superscript') {
+                                $format_text{Super} = 1;
+                            }
+                            elsif ($node_format->att('val') eq 'subscript') {
+                                $format_text{Super} = 2;
+                            }
+                        }
+                    }
                     push @rich_font_by_cell, [
                         $string_length,
-                        Spreadsheet::ParseExcel::Font->new(%default_font_opts)
+                        Spreadsheet::ParseExcel::Font->new(%format_text)
                     ];
                 }
-                $string_text .= $chunk->text;
-            }
-            elsif ($chunk->name eq 's:rPr') {
-                my %format_text = %default_font_opts;
-                for my $node_format ($chunk->children) {
-                    if ($node_format->name eq 's:sz') {
-                        $format_text{Height} = $node_format->att('val');
-                    }
-                    elsif ($node_format->name eq 's:color') {
-                        $format_text{Color} = $self->_color(
-                            $theme_colors,
-                            $node_format
-                        );
-                    }
-                    elsif ($node_format->name eq 's:rFont') {
-                        $format_text{Name} = $node_format->att('val');
-                    }
-                    elsif ($node_format->name eq 's:b') {
-                        $format_text{Bold} = 1;
-                    }
-                    elsif ($node_format->name eq 's:i') {
-                        $format_text{Italic} = 1;
-                    }
-                    elsif ($node_format->name eq 's:u') {
-                        $format_text{Underline} = 1;
-                        if (defined $node_format->att('val')) {
-                            $format_text{UnderlineStyle} = 2;
-                        } else {
-                            $format_text{UnderlineStyle} = 1;
-                        }
-                    }
-                    elsif ($node_format->name eq 's:strike') {
-                        $format_text{Strikeout} = 1;
-                    }
-                    elsif ($node_format->name eq 's:vertAlign') {
-                        if ($node_format->att('val') eq 'superscript') {
-                            $format_text{Super} = 1;
-                        }
-                        elsif ($node_format->att('val') eq 'subscript') {
-                            $format_text{Super} = 2;
-                        }
-                    }
-                }
-                push @rich_font_by_cell, [
-                    $string_length,
-                    Spreadsheet::ParseExcel::Font->new(%format_text)
-                ];
             }
         }
-    }
-    else {
-        $string_text = join '', map { $_->text } $si->find_nodes('.//s:t');
+        else {
+            # $subnode->name is either 's:rPh' or 's:phoneticPr'
+            # We ignore phonetic information and do nothing.
+        }
     }
 
     return (
@@ -929,6 +1045,29 @@ sub _extract_files {
         ($_->att('Id') => $self->_zip_file_member($zip, $get_path->($_->att('Target'))))
     } $wb_rels->find_nodes(qq<//packagerels:Relationship[\@Type="$type_base/worksheet"]>);
 
+    # If we have hyperlinks in cells we need the rels file to get the link details
+    my $worksheet_rels_xml;
+
+    # Get each worksheet object
+    foreach my $worksheet ($wb_rels->find_nodes(qq<//packagerels:Relationship[\@Type="$type_base/worksheet"]>)) {
+        # Split the worksheet xml path so we can
+        my @sheetname_parts = split('/', $worksheet->att('Target'));
+
+        # Insert _rels before the sheetname, and amend the filename to have .rels on the end
+        my $sheetname = pop(@sheetname_parts);
+        push(@sheetname_parts, '_rels');
+        push(@sheetname_parts, $sheetname . '.rels');
+
+        # Recreate the file path
+        my $rels_name = join('/', @sheetname_parts);
+
+        # Check if we have a rels file
+        if (my $relfile = $zip->memberNamed($get_path->($rels_name)) ) {
+            # Add the XML to our hash for access later on
+            $worksheet_rels_xml->{$worksheet->att('Id')} = $relfile->contents;
+        }
+    }
+
     my %themes_xml = map {
         $_->att('Id') => $self->_parse_xml($zip, $get_path->($_->att('Target')))
     } $wb_rels->find_nodes(qq<//packagerels:Relationship[\@Type="$type_base/theme"]>);
@@ -943,6 +1082,7 @@ sub _extract_files {
         ($strings_xml
             ? (strings => $strings_xml)
             : ()),
+        (($worksheet_rels_xml && keys(%$worksheet_rels_xml)) ? (sheets_rels => $worksheet_rels_xml) : ()),
     };
 }
 
@@ -1129,7 +1269,7 @@ Spreadsheet::ParseXLSX - parse XLSX files
 
 =head1 VERSION
 
-version 0.28
+version 0.29
 
 =head1 SYNOPSIS
 
@@ -1286,7 +1426,7 @@ Jesse Luehrs <doy@tozt.net>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alexey Mazurin Dave Clarke Fitz Elliott FL Meredith Howard MichaelDaum rdboisvert Ricardo Signes Robert Boisvert Steve Simms Stuart Watt zhouzhen1
+=for stopwords Alexey Mazurin Daryl Anderman Dave Clarke Fitz Elliott FL Gareth Tunley Meredith Howard Michael Daum Nick Moore oharakouhei rdboisvert Ricardo Signes Robert Boisvert Steve Simms Stuart Watt theevilapplepie zhouzhen1
 
 =over 4
 
@@ -1296,6 +1436,10 @@ Alexey Mazurin <mazurin.alexey@gmail.com>
 
 =item *
 
+Daryl Anderman <daryl@273kelvin.ca>
+
+=item *
+
 Dave Clarke <david_clarke@verizon.net>
 
 =item *
@@ -1308,11 +1452,23 @@ FL <f20@reckon.co.uk>
 
 =item *
 
+Gareth Tunley <gjtunley@gmail.com>
+
+=item *
+
 Meredith Howard <mhoward@roomag.org>
 
 =item *
 
-MichaelDaum <daum@michaeldaumconsulting.com>
+Michael Daum <daum@michaeldaumconsulting.com>
+
+=item *
+
+Nick Moore <nick.moore@strategicdata.com.au>
+
+=item *
+
+oharakouhei <gofue0723@gmail.com>
 
 =item *
 
@@ -1336,6 +1492,10 @@ Stuart Watt <stuart@morungos.com>
 
 =item *
 
+theevilapplepie <theevilapplepie@gmail.com>
+
+=item *
+
 zhouzhen1 <zhouzhen1@gmail.com>
 
 =back
