@@ -3,7 +3,7 @@ package Spreadsheet::ParseXLSX::Decryptor;
 use strict;
 use warnings;
 
-our $VERSION = '0.28'; # VERSION
+our $VERSION = '0.29'; # VERSION
 
 # ABSTRACT: helper class to open password protected files
 
@@ -148,6 +148,7 @@ sub _agileDecryption {
     my $encryptedVerifierHashInput = MIME::Base64::decode($info->att('encryptedVerifierHashInput'));
     my $encryptedVerifierHashValue = MIME::Base64::decode($info->att('encryptedVerifierHashValue'));
     my $encryptedKeyValue = MIME::Base64::decode($info->att('encryptedKeyValue'));
+    my $hashSize = 0 + $info->att('hashSize');
 
     my $keyDecryptor = Spreadsheet::ParseXLSX::Decryptor::Agile->new({
                   cipherAlgorithm => $info->att('cipherAlgorithm'),
@@ -160,7 +161,7 @@ sub _agileDecryption {
                   blockSize       => 0 + $info->att('blockSize')
               });
 
-    $keyDecryptor->verifyPassword($encryptedVerifierHashInput, $encryptedVerifierHashValue);
+    $keyDecryptor->verifyPassword($encryptedVerifierHashInput, $encryptedVerifierHashValue, $hashSize);
 
     my $key = $keyDecryptor->decrypt($encryptedKeyValue, "\x14\x6e\x0b\xe7\xab\xac\xd0\xd6");
 
@@ -199,7 +200,7 @@ sub new {
 
     if ($self->{hashAlgorithm} eq 'SHA512') {
         $self->{hashProc} = \&Digest::SHA::sha512;
-    } elsif ($self->{hashAlgorithm} eq 'SHA-1') {
+    } elsif (($self->{hashAlgorithm} eq 'SHA-1') || ($self->{hashAlgorithm} eq 'SHA1')) {
         $self->{hashProc} = \&Digest::SHA::sha1;
     } elsif ($self->{hashAlgorithm} eq 'SHA256') {
         $self->{hashProc} = \&Digest::SHA::sha256;
@@ -225,7 +226,7 @@ Spreadsheet::ParseXLSX::Decryptor - helper class to open password protected file
 
 =head1 VERSION
 
-version 0.28
+version 0.29
 
 =for Pod::Coverage   new
   open
