@@ -3,7 +3,7 @@ package Spreadsheet::ParseXLSX::Decryptor::Agile;
 use strict;
 use warnings;
 
-our $VERSION = '0.28'; # VERSION
+our $VERSION = '0.29'; # VERSION
 
 # ABSTRACT: decryptor for files of version 4.4
 
@@ -94,10 +94,12 @@ sub decryptFile {
 
 sub verifyPassword {
     my $self = shift;
-    my ($encryptedVerifier, $encryptedVerifierHash) = @_;
+    my ($encryptedVerifier, $encryptedVerifierHash, $hashSize) = @_;
 
     my $encryptedVerifierHash0 = $self->{hashProc}->($self->decrypt($encryptedVerifier, "\xfe\xa7\xd2\x76\x3b\x4b\x9e\x79"));
     $encryptedVerifierHash = $self->decrypt($encryptedVerifierHash, "\xd7\xaa\x0f\x6d\x30\x61\x34\x4e");
+    $encryptedVerifierHash0 = substr($encryptedVerifierHash0, 0, $hashSize);
+    $encryptedVerifierHash = substr($encryptedVerifierHash, 0, $hashSize);
 
     die "Wrong password: $self" unless ($encryptedVerifierHash0 eq $encryptedVerifierHash);
 }
@@ -117,7 +119,7 @@ Spreadsheet::ParseXLSX::Decryptor::Agile - decryptor for files of version 4.4
 
 =head1 VERSION
 
-version 0.28
+version 0.29
 
 =for Pod::Coverage   decrypt
   decryptFile
